#!/usr/bin/perl

#    Copyright (C) 2000 Sinclair InterNetworking Services Pty Ltd
#    <nmis@sins.com.au> http://www.sins.com.au/nmis

use strict;

# Variables for command line munging
my %ARG = getArguements(@ARGV);

if ( ! defined $ARG{dir} ) {
	$ARG{dir} = "/usr/local/nmis";
} 

if ( ! defined $ARG{perl} ) {
	$ARG{perl} = "\#!/usr/local/bin/perl";
} 

if ( $ARG{perl} =~ /\#!(.*)/ ) {
	print "Removing \#! from $ARG{perl}\n";
	$ARG{perl} = $1;
}

my $debug = &setDebug($ARG{debug});

my $pass = 0;
my $dirpass = 1;
my $dirlevel = 0;
my $maxrecurse = 200;
my $maxlevel = 10;

my $bad_file;
my $bad_dir;
my $file_count;
my $extension = "pl";

print "Changing all Perl \#! line to $ARG{perl}.\n";

&processDir(dir => $ARG{dir});

print "Done.  Processed $file_count perl files.\n";

exit 1;

sub processDir {
	my %args = @_;
	# Starting point
	my $dir = $args{dir};
	my @dirlist;	
	my $index;
	++$dirlevel;
	my @filename; 
	my $key;

	if ( -d $dir ) {
		print "\nProcessing Directory $dir pass=$dirpass level=$dirlevel\n"; 
	}
	else {
		print "\n$dir is not a directory\n";
		exit -1;
	}	

	#sleep 1;
	if ( $dirpass >= 1 and $dirpass < $maxrecurse and $dirlevel <= $maxlevel ) {
		++$dirpass;
		opendir (DIR, "$dir");
		@dirlist = readdir DIR;
		closedir DIR;
		
		if ($debug > 1) { print "\tFound $#dirlist entries\n"; }
		
		for ( $index = 0; $index <= $#dirlist; ++$index ) {
			@filename = split(/\./,"$dir/$dirlist[$index]");
			if ( -f "$dir/$dirlist[$index]"
				and $extension =~ /$filename[$#filename]/i 
				and $bad_file !~ /$dirlist[$index]/i
			) {
				if ($debug>1) { print "\t\t$index file $dir/$dirlist[$index]\n"; }
				&backupFile(file => "$dir/$dirlist[$index]", backup => "$dir/$dirlist[$index].bak" )
				&processPerlFile(file => "$dir/$dirlist[$index]", perl => $ARG{perl} )
			}
			elsif ( -d "$dir/$dirlist[$index]" 
				and $dirlist[$index] !~ /^\./ 
				and $bad_dir !~ /$dirlist[$index]/i
			) {
				#if (!$debug) { print "."; }
				&processDir(dir => "$dir/$dirlist[$index]");
				--$dirlevel;
			}
		}	
	}
} # processDir

sub processPerlFile {
	my %args = @_;
	++$file_count;
	my @lines;
	my $countlines=0 ;
	if ($debug) { print "Processing $args{file}\n"; }
	open(IN, $args{file}) or warn "ERROR with $args{file}. $!\n";
	while (<IN>) {
		push (@lines,$_);		
	} # while IN
	close(IN);

	open(OUT, ">$args{file}") or warn "ERROR with $args{file}. $!\n";
	foreach (@lines) {
		$countlines++ ;
		$_ = rmTrailingBits($_);
		if ($countlines==1) {
			if ( $_ =~ /^\#!/ ) {
				$_ =~ s/\n//;
				print "$args{file} changing $_ to $args{perl}\n";
				print OUT "\#!$args{perl}\n";
			}
		} 
		else {
			print OUT $_;
		}
	} # while IN
	close(OUT);
}

sub getArguements {
	my @argue = @_;
	my (%nvp, $name, $value, $line, $i);
	for ($i=0; $i <= $#argue; ++$i) {
	        if ($argue[$i] =~ /.+=/) {
	                ($name,$value) = split("=",$argue[$i]);
	                $nvp{$name} = $value;
	        } 
	        else { print "Invalid command argument: $argue[$i]\n"; }
	}
	return %nvp;
}

sub setDebug {
	my $string = shift;
	my $debug = 0;
	if ( $string eq "true" ) { $debug = 1; }	
	elsif (  $string eq "verbose" ) { $debug = 9; }	
	elsif ( $string =~ /\d+/ ) { $debug = $string; }	
	else { $debug = 0; }	
	return $debug;
}

sub rmTrailingBits {
	my $string = shift;
	while ( $string =~ /\r/ ) {
		$string =~ s/\r//g;
	}
	return $string;
}

sub backupFile {
	my %arg = @_;
	my $buff;
	if ( -r $arg{file} ) {
		open(IN,$arg{file}) or warn ("ERROR: problem with file $arg{file}; $!");
		open(OUT,">$arg{backup}") or warn ("ERROR: problem with file $arg{backup}; $!");
		binmode(IN);
		binmode(OUT);
		while (read(IN, $buff, 8 * 2**10)) {
		    print OUT $buff;
		}
		close(IN);
		close(OUT);
		return 1;
	} else {
		print STDERR "ERROR, backupFile file $arg{file} not readable.\n";
		return 0;
	}
}
