#!/usr/bin/perl
#
#    csv-append.pl - NMIS Perl Program - Network Mangement Information System
#    Copyright (C) 2000 Sinclair InterNetworking Services Pty Ltd
#    <nmis@sins.com.au> http://www.sins.com.au/nmis
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# used to change or append a new element to a .csv file
# write a new csv file (.new) for checking before use
# uses the pre-defined table names in nmis.conf
# for example
# ./csv-append.pl table=Nodes element=depend value=N/A
#
# can also be used to change or set all the values of a table element to a value
# *****************************************************************************

# Auto configure to the <nmis-base>/lib and <nmis-base>/files/nmis.conf
use FindBin;
use lib "$FindBin::Bin/../lib";
# 
# ****** Shouldn't be anything else to customise below here *******************
#

require 5;

use strict;
use csv;
use NMIS;
use func;

# Check if there are any arguements
if ( $#ARGV < 0 ) {
        print "$0 NMIS CSV Append Tool\n";
        print "command line options are:\n";
        print "\tfile=<[nmis.conf]> Which nmis.conf file to use - defaults ../conf/nmis.conf\n";
        print "\ttable=<Tablename>  - Name of Table in nmis.conf\n";
        print "\tfield=<xxx> - CSV field to be appended or changed\n";
		print "\tvalue=<xxx> - field value\n";
        print "\n";
        exit(1);
}

my %ARGS = getArguements(@ARGV);

my $conf;
if ( $ARGS{file} ne "" ) { $conf = $ARGS{file}; }
else { $conf = "nmis.conf"; }
my $configfile = "$FindBin::Bin/../conf/$conf";
if ( -f $configfile ) { loadConfiguration($configfile); }
# the following should be conformant to Linux FHS
elsif ( -e "/etc/nmis/$conf") { loadConfiguration ("/etc/nmis/$conf");}
else { die "Can't access neither configuration file $configfile, nor /etc/nmis/$conf \n"; }
								       
my $table = $ARGS{table}."_Table";
my $key = $ARGS{table}."_Key";
my $outfile = $NMIS::config{$table}.".new";

die "Cannot read $table\n\n" unless -r $NMIS::config{$table};

my %hash = &loadCSV("$NMIS::config{$table}","$NMIS::config{$key}","\t");

foreach ( keys %hash ) {
	$hash{$_}{$ARGS{field}} = $ARGS{value};
}

&writeCSV(%hash,"$outfile","\t");
print "File $outfile written\nCheck file permissions and ownership before using !\n\n";

