#!/usr/bin/perl
use 5.005;

# this is run as a daemon on the Master
# receives reach rrd updates from slave(s) to populate master with basic reachibility and health stats
# means that runmetrics on master is then summary for entire network yippe !!
# also 'Node Down' posted to here - just write it to slave_node_event.dat, so we can colour the master display

# Initial Setup

#require Data::Dumper;
#Data::Dumper->import();
#$Data::Dumper::Indent = 1;

#! this imports the LOCK_ *constants (eg. LOCK_UN, LOCK_EX)
use Fcntl qw(:DEFAULT :flock);
	
# Auto configure to the <nmis-base>/lib and <nmis-base>/files/nmis.conf
use FindBin;
use lib "$FindBin::Bin/../lib";
use lib "/usr/local/rrdtool/lib/perl"; 

use RRDs;
use NMIS;
use rrdfunc;
use Time::HiRes qw(sleep);

my $configfile = "$FindBin::Bin/../conf/nmis.conf";
if ( -r $configfile ) { loadConfiguration($configfile); }
# the following should be conformant to Linux FHS
elsif ( -e "/etc/nmis/$conf") { loadConfiguration ("/etc/nmis/$conf");}
else { die "Can't access neither configuration file $configfile, nor /etc/nmis/$conf \n"; }

# Get command line arguments

($VARDIR, $PORT ) = @ARGV;
unless (-d $VARDIR) { $VARDIR = $NMIS::config{'<nmis_var>'} }
unless ($PORT) { $PORT = '2345' }

# Initialize globals

$RESTARTCMD = "$0 '$VARDIR' '$PORT'";

# See if another instance of ourself is running, if so kill the process

$PIDFILE = $VARDIR."/master.pid";

if (-f $PIDFILE) {
  open(F, "<$PIDFILE");
  $pid = <F>;
  close(F);
  chomp $pid;
  if ($pid != $$) {
    &DEBUG("Pidfile exists, killing the pidfile process $pid");
    kill 9, $pid;
    unlink($PIDFILE);
    &DEBUG("$PIDFILE deleted");
  }
}

# daemonise ourselves.

fork && exit;  
close(STDIN); close(STDERR); close(STDOUT);
use POSIX;
POSIX::setsid();
&DEBUG('*** Forked process: ',$$);
# rename ourselves
$0 = 'master.pl';

# ---- Subroutines ---- #

sub DEBUG {
	logMessage("master.pl, ,".join("",@_) );
}


sub signal_handler {
    $time_to_die = 1;                           # Exits the main "until" loop
    $time_to_restart = 0;                       # We don't want to restart ourself
    &DEBUG('TERM or INT signal received.');
}

sub signal_hup {
    $time_to_die = 1;                           # Exit the main "until" loop
    $time_to_restart = 1;                       # We want to restart ourself
    &DEBUG('HUP signal received.');
}

# ---- End Subroutines ---- #

# Now for the main code.
# First make sure we trap the signals sent to us.

$SIG{INT} = $SIG{TERM} = \&signal_handler;
$SIG{HUP} = \&signal_hup;

# Announce our presence via a PID file

open(PID, ">$PIDFILE") || exit;
print PID $$; close(PID);
&DEBUG("pidfile $PIDFILE created");

# Perform a sanity check. If the current PID file is not the same as
# our PID then we have become detached somehow, so just exit

open(PID, "<$PIDFILE") || exit;
$pid = <PID>; close(PID);
chomp $pid;
exit unless $pid == $$;

# Record our (re)starting in the event log

&DEBUG("start: pid=$PIDFILE port=$PORT");

# code body here.

use Net::EasyTCP;

#
# Create the server object
#
$server = new Net::EasyTCP(
        mode            =>      "server",
	host		=>	"127.0.0.1",
        port            =>      2345,
        donotencrypt	=>		1,
        donotcompress	=>		1
)
|| die "ERROR CREATING SERVER: $@\n";

#
# Tell it about the callbacks to call
# on known events
#
$server->setcallback(
        data            =>      \&gotdata,
        connect         =>      \&connected,
        disconnect      =>      \&disconnected,
)
|| die "ERROR SETTING CALLBACKS: $@\n";

#
# This sub gets called when a client sends us data
#
sub gotdata {
    my $client = shift;
    my $serial = $client->serial();
    my $data = $client->data();		# this is a hashref

    # a sanity check....
    # my $size = keys(%{$data});
	# always at least 1 hostname record plus a varying amount of node records
	# send to our log for traceability recording.
	# logMessage("master.pl, received $size records of info from $$data{hostname}\n");

    
	# lets save some data away
	# here we only expect reachability rrd and slave events. iF you want more - just write it in
	foreach my $node ( keys %{$data} ) {
		if ( $$data{$node}{option} eq 'RRDDB' and $$data{$node}{rrd} eq 'reach' ) {	
			&updateRRDDBreach(node => $node, nodetype => $$data{$node}{devicetype}, data => $$data{$node} ); 
		}
	}
	# now check for events to write into slave event.dat file.
	# only write node down events to colorise master display - in fact this is all that we receive anyway !
	sysopen(FILE, "$NMIS::config{'<nmis_var>'}/$$data{hostname}_event.dat", O_RDWR | O_CREAT ) or warn "master.pm, can't open filename: $!";
	flock(FILE, LOCK_EX) or warn "master.pm, can't lock filename: $!";
	# rewind and clear the file, as we are going to write back the current events - or none if none..
	seek(FILE,0,0) or warn "master.pm, can't seek filename: $!";
	truncate(FILE, 0) or warn "master.pm, can't truncate filename: $!";

	foreach my $node ( keys %{$data} ) {
		if ( exists $$data{$node}{event} ) {
			# Update the event file
			print FILE "$$data{$node}{event}{startdate},$$data{$node}{event}{lastchange},$$data{$node}{event}{node},$$data{$node}{event}{event},$$data{$node}{event}{event_level},$$data{$node}{event}{details},$$data{$node}{event}{ack},$$data{$node}{event}{escalate},$$data{$node}{event}{notify}\n";
		}
	}
	# if no nodes down - we should be left with an empty file
	close(FILE) or warn "master.pm, can't close filename: $!";
	
	# set file owner and permission, default: nmis, 0775.
	setFileProt("$NMIS::config{'<nmis_var>'}/$$data{hostname}_event.dat"); # Cologne, Jan 2005

}

#
# This sub gets called when a new client connects
#
sub connected {
        #my $client = shift;
        #my $serial = $client->serial();
        #my $ip = $client->remoteip();
        #&DEBUG("Client $ip:$serial just connected");
}

#
# This sub gets called when an existing client disconnects
#
sub disconnected {
        #my $client = shift;
        #my $serial = $client->serial();
        #my $ip = $client->remoteip();
        #&DEBUG("Client $ip:$serial just disconnected");
}


# Main Loop. Continue until we receive a signal to stop or restart
until ($time_to_die) {
	# Start the server and come back here in case we received a signal.
	#
	$server->do_one_loop() || die "ERROR STARTING SERVER: $@\n";
	sleep (0.25);
}

# stop the server

$server->stop();

# It's time to exit. First, delete the PID and any LOCK files.

unlink($PIDFILE);
&DEBUG("pidfile $PIDFILE deleted");

# If we were asked to restart then do so, otherwise we are done.

if ($time_to_restart) {
  &DEBUG("Restarting: ",$RESTARTCMD);
  exec($RESTARTCMD);
}
&DEBUG('Exiting');
exit;


# ** subs ***

# this will update the DS as required - maybe a little scary for the master if a slave chucks a 100 or so at us at once for update....
# I think a manual collect run on the slave, and wait for the master to update all would be better...

sub updateRRDDBreach {
	my %arg = @_;
	my $data = $arg{data};
	my $i;
	my @label;
	my @value;

	my @options;
	my $ERROR;

	# check if RRD database exist, else create
	if ( not &createRRDDBreach(node => $arg{node}, nodetype => $arg{nodetype} ) ) {
		return;
	}
	
	#'reach' => "/health/$arg{nodetype}/$arg{node}-reach.rrd
	my $database = "$NMIS::config{database_root}/health/$arg{nodetype}/$arg{node}-reach.rrd";

	@options = (
		"-t", "reachability:availability:responsetime:health:loss",
		"N:$$data{reachability}:$$data{availability}:$$data{responsetime}:$$data{health}:$$data{loss}"
	);

	# update RRD
	RRDs::update($database,@options);
	if ($ERROR = RRDs::error) {
		if ($ERROR !~ /Template contains more DS/) {
			&DEBUG("updateRRD, $node, update ERROR database=$database: $ERROR: options = @options");
		} else {
			&DEBUG(" updateRRD: missing DataSource in $database, try to update\n");
			# find the DS names in the existing database (format ds[name].* )
			my $info = RRDs::info($database);
			my $names = ":";
			foreach my $key (keys %$info) {
				if ( $key =~ /^ds\[([a-zA-Z0-9_]{1,19})\].+/) { $names .= "$1:";}
			}
			# find the missing DS name (format DS:name:type:hearthbeat:min:max)
			my @options_db = &optionsReach();
			foreach my $ds (@options_db) {
				my @opt = split /:/, $ds;
				if ( $opt[0] eq "DS" and $names !~ /:$opt[1]:/ ) {
					&addDStoRRD($database,$ds); # sub in rrdfunc
				}
			}
		}
	}
}


sub createRRDDBreach {
	my %arg = @_;
	
	#'reach' => "/health/$arg{nodeType}/$arg{node}-reach.rrd
	my $database = "$NMIS::config{database_root}/health/$arg{nodetype}/$arg{node}-reach.rrd";

	# Does the database exist already? Just exit true if so
	if ( -f $database and -r $database and -w $database ) { 
		return 1;
	}
	# Check if the RRD Database Exists but is ReadOnly
	# Maybe this should check for valid directory or not.
	elsif ( -f $database and not -w $database ) { 
		&DEBUG( "createRRDDBreach: Database $database Exists but is readonly to you!");
		return 0;
	}
	# It doesn't so create it
	else {

		my @options = &optionsReach();
		my $time  = 30*int(time/30);

		RRDs::create("$database",@options);
		my $ERROR = RRDs::error;
		if ($ERROR) {
			&DEBUG("createRRDDBreach: unable to create $database: $ERROR");
			return 0;
		}
		# Double check created OK for this user
		if ( -f $database and -r $database and -w $database ) { 
			&DEBUG("createRRDDBreach, Created RRD $database starting at $time");
			
			# set file owner and permission, default: nmis, 0775.
			setFileProt($database); # Cologne, Jan 2005
			return 1;
		}
		else {
			&DEBUG("createRRDDBreach: Could not create RRD $database - check directory permissions");
			return 0;
		}
	} # else
} # end createRRDDBreach


sub optionsReach {

	my $time  = 30*int(time/30);
	my @options;
	my $START = $time;

	@options = (
		"-b", $START, "-s", $NMIS::config{RRD_poll},
		"DS:reachability:GAUGE:$NMIS::config{RRD_hbeat}:0:100", 
		"DS:availability:GAUGE:$NMIS::config{RRD_hbeat}:0:100", 
		"DS:responsetime:GAUGE:$NMIS::config{RRD_hbeat}:0:U", 
		"DS:health:GAUGE:$NMIS::config{RRD_hbeat}:0:100",
		"DS:loss:GAUGE:$NMIS::config{RRD_hbeat}:0:100",
		"RRA:AVERAGE:0.5:$NMIS::config{RRA_step_int_day}:$NMIS::config{RRA_rows_rch_day}",
		"RRA:AVERAGE:0.5:$NMIS::config{RRA_step_int_week}:$NMIS::config{RRA_rows_rch_week}",
		"RRA:AVERAGE:0.5:$NMIS::config{RRA_step_int_month}:$NMIS::config{RRA_rows_rch_month}",
		"RRA:AVERAGE:0.5:$NMIS::config{RRA_step_int_year}:$NMIS::config{RRA_rows_rch_year}",
		"RRA:MAX:0.5:$NMIS::config{RRA_step_int_day}:$NMIS::config{RRA_rows_rch_day}",
		"RRA:MAX:0.5:$NMIS::config{RRA_step_int_week}:$NMIS::config{RRA_rows_rch_week}",
		"RRA:MAX:0.5:$NMIS::config{RRA_step_int_month}:$NMIS::config{RRA_rows_rch_month}",
		"RRA:MAX:0.5:$NMIS::config{RRA_step_int_year}:$NMIS::config{RRA_rows_rch_year}",
		"RRA:MIN:0.5:$NMIS::config{RRA_step_int_day}:$NMIS::config{RRA_rows_rch_day}",
		"RRA:MIN:0.5:$NMIS::config{RRA_step_int_week}:$NMIS::config{RRA_rows_rch_week}",
		"RRA:MIN:0.5:$NMIS::config{RRA_step_int_month}:$NMIS::config{RRA_rows_rch_month}",
		"RRA:MIN:0.5:$NMIS::config{RRA_step_int_year}:$NMIS::config{RRA_rows_rch_year}"
	);
	return (@options);
}
