#!/usr/bin/perl
# used for creating individual oid.files from a mib file
# Auto configure to the <nmis-base>/lib and <nmis-base>/files/nmis.conf
use FindBin;
use lib "$FindBin::Bin/../lib";

# Check if there are any arguements
if ( $#ARGV < 0 ) {
        print "$0 NMIS MIB Dumping Tool\n";
        print "command line options are:\n";
        print "\tmibdir=<directory>     Location of the MIBS\n";
        print "\tmib=<mibfile>  Which MIB to load\n";
        print "\toutfile=<filename>]    Where to dump the OIDs to.\n";
        print "\n";
        
        exit(1);
}

use strict;
use SNMP_MIB;
use func;
use NMIS;

$SNMP_MIB::suppress_warnings=0;

my %ARGS = getArguements(@ARGV);

my $conf;
if ( $ARGS{file} ne "" ) { $conf = $ARGS{file}; }
else { $conf = "nmis.conf"; }
my $configfile = "$FindBin::Bin/../conf/$conf";
if ( -f $configfile ) { loadConfiguration($configfile); }
# the following should be conformant to Linux FHS
elsif ( -e "/etc/nmis/$conf") { loadConfiguration ("/etc/nmis/$conf");}
else { die "Can't access neither configuration file $configfile, nor /etc/nmis/$conf \n"; }

if ( ! defined $ARGS{mibdir} ) { $ARGS{mibdir} = "/usr/local/share/snmp/mibs/"; }

# Always load the default mibs from the master mib dir
SNMP_MIB::loadmib($ARGS{mibdir}, "RFC1213-MIB.txt");
SNMP_MIB::loadmib($ARGS{mibdir}, "IF-MIB.txt");
SNMP_MIB::loadmib($ARGS{mibdir}, "IANAifType-MIB-V1SMI.txt");
SNMP_MIB::loadmib($ARGS{mibdir}, "ETHERLIKE-MIB.txt");
SNMP_MIB::loadmib($ARGS{mibdir}, "RFC1315-MIB.txt");
SNMP_MIB::loadmib($ARGS{mibdir}, "ENTITY-MIB-V1SMI.txt");
SNMP_MIB::loadmib($ARGS{mibdir}, "SNMPv2-SMI.txt");
SNMP_MIB::loadmib($ARGS{mibdir}, "SNMPv2-MIB.txt");
SNMP_MIB::loadmib($ARGS{mibdir}, "SNMPv2-TC.txt");
SNMP_MIB::loadmib($ARGS{mibdir}, "SNMPv2-CONF.txt");

# expect fully pathed directory here.
SNMP_MIB::loadmib($ARGS{mibdir},$ARGS{mib});

# current dir here
open (STDOUT,">$ARGS{outfile}") or die "ERROR, problem opening $ARGS{outfile}. $!\n";
SNMP_MIB::dump_oids_file();
close (STDOUT);


