#!/usr/bin/perl
#
#    mibdump.pl - NMIS CGI Program - Network Mangement Information System
#    Copyright (C) 2000 Sinclair InterNetworking Services Pty Ltd
#    <nmis@sins.com.au> http://www.sins.com.au/nmis
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#*****************************************************************************
# Auto configure to the <nmis-base>/lib and <nmis-base>/files/nmis.conf
use FindBin;
use lib "$FindBin::Bin/../lib";

#****** Shouldn't be anything else to customise below here *******************

# Check if there are any arguements
if ( $#ARGV < 0 ) {
        print "$0 NMIS MIB Dumping Tool\n";
        print "command line options are:\n";
        print "\tmibdir=<directory>     Location of the MIBS\n";
        print "\tmibs=<all|standard|router|switch|server|a3com>]  Which MIBs to load\n";
        print "\toutfile=<filename>]    Where to dump the OIDs to.\n";
        print "\n";
        
        exit(1);
}

use strict;
use SNMP_MIB;
use func;
use NMIS;

my %ARGS = getArguements(@ARGV);

my $conf;
if ( $ARGS{file} ne "" ) { $conf = $ARGS{file}; }
else { $conf = "nmis.conf"; }
my $configfile = "$FindBin::Bin/../conf/$conf";
if ( -f $configfile ) { loadConfiguration($configfile); }
# the following should be conformant to Linux FHS
elsif ( -e "/etc/nmis/$conf") { loadConfiguration ("/etc/nmis/$conf");}
else { die "Can't access neither configuration file $configfile, nor /etc/nmis/$conf \n"; }
								       
if ( ! defined $ARGS{mibdir} ) { $ARGS{mibdir} = "/nmis/netsnmp/share/snmp/mibs"; }

$SNMP_Simple::suppress_warnings=0;
$SNMP_Session::suppress_warnings=0;
$SNMP_MIB::suppress_warnings=0;


# Always load the default mib
SNMP_MIB::loadmib($ARGS{mibdir}, "RFC1213-MIB.txt");

&loadMibs();
&dumpOID();

sub loadMibs {
	print "Loading MIBS from $ARGS{mibdir}\n";
	if ( $ARGS{mibs} eq "all" ) { 
		# Standard Mibs
		# EHG 31/8/02 altered to standard load order
		# edited cisco-smi to fix ciscoProxy warning
		SNMP_MIB::loadmib($ARGS{mibdir}, "SNMPv2-SMI-V1SMI.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "SNMPv2-TC-V1SMI.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "SNMPv2-MIB-V1SMI.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "IF-MIB-V1SMI.txt");		
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-SMI-V1SMI.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-PRODUCTS-MIB-V1SMI.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-TC.txt");
                SNMP_MIB::loadmib($ARGS{mibdir}, "IF-MIB.txt");         # RFC2233 HC counter extensions
		SNMP_MIB::loadmib($ARGS{mibdir}, "IANAifType-MIB-V1SMI.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "ETHERLIKE-MIB.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "RFC1315-MIB.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "BRIDGE-MIB.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "ENTITY-MIB-V1SMI.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-STACK-MIB-V1SMI.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-PROCESS-MIB-V1SMI.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-MEMORY-POOL-MIB-V1SMI.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-ENVMON-MIB-V1SMI.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-FRAME-RELAY-MIB-V1SMI.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-FIREWALL-MIB-V1SMI.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "OLD-CISCO-SYS-MIB.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "OLD-CISCO-CHASSIS-MIB.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "OLD-CISCO-INTERFACES-MIB.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "UCD-SNMP-MIB.txt");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "sun.mib");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "snmpdx.mib");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "msft.mib");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "lmmib2.mib");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "MS-Perfmib.mib");
		SNMP_MIB::loadmib($ARGS{mibdir}, "HOST-RESOURCES-MIB.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "HOST-RESOURCES-TYPES.txt");
	}
	elsif ( $ARGS{mibs} eq "standard" ) { 
		# Standard Mibs
		SNMP_MIB::loadmib($ARGS{mibdir}, "IF-MIB-V1SMI.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "IANAifType-MIB-V1SMI.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "ETHERLIKE-MIB.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "RFC1315-MIB.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "ENTITY-MIB-V1SMI.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "SNMPv2-SMI-V1SMI.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "SNMPv2-MIB-V1SMI.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "IF-MIB.txt");		# RFC2233 HC counter extensions
		
	}
	elsif ( $ARGS{mibs} eq "switch" ) { 
		# Cisco Switch MIBS
		SNMP_MIB::loadmib($ARGS{mibdir}, "BRIDGE-MIB.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-SMI-V1SMI.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-PRODUCTS-MIB-V1SMI.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-STACK-MIB-V1SMI.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-PROCESS-MIB-V1SMI.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-MEMORY-POOL-MIB-V1SMI.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-ENVMON-MIB-V1SMI.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "OLD-CISCO-INTERFACES-MIB.txt");
		# Nortel MIBS?
		#SNMP_MIB::loadmib($ARGS{mibdir}, "ACCELAR-MIB.txt");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "SYNRO172-MIB.txt");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "S5REG180-MIB.txt");

	}
	elsif ( $ARGS{mibs} eq "router" ) { 
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-SMI-V1SMI.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-PRODUCTS-MIB-V1SMI.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "OLD-CISCO-SYS-MIB.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "OLD-CISCO-CHASSIS-MIB.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "OLD-CISCO-INTERFACES-MIB.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-PROCESS-MIB-V1SMI.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-MEMORY-POOL-MIB-V1SMI.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-ENVMON-MIB-V1SMI.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-FRAME-RELAY-MIB-V1SMI.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-FIREWALL-MIB-V1SMI.txt");
	}	
	elsif ( $ARGS{mibs} eq "server" ) { 
		# Server MIBS
		SNMP_MIB::loadmib($ARGS{mibdir}, "UCD-SNMP-MIB.txt");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "sun.mib");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "sun-snmpdx.mib");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "MS-LMMIB2.MIB");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "MS-Perfmib.mib");
		SNMP_MIB::loadmib($ARGS{mibdir}, "HOST-RESOURCES-MIB.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "dellserv.mib");

	}
	elsif ( $ARGS{mibs} eq "a3com" ) { 
		# 3com Switch MIBS
		# as of 3.0, only using lbhub-boom.mib
		# rest here for future use
		#SNMP_MIB::loadmib($ARGS{mibdir}, "3c16640.mib");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "3com-products.mib");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "3com.oid");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "a3com-bridge");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "a3com-filter");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "a3com-ip");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "a3com-ip-rip");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "a3com-ipso");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "a3com-ipx");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "a3com-ipxpolicy");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "a3com-port.path");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "a3com-sys");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "alert.mib");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "audlmib");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "dlswmib.br83");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "dvmrpmib");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "fms100.mib");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "isd.mib");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "lbhub-blc.mib");
		SNMP_MIB::loadmib($ARGS{mibdir}, "lbhub-boom.mib");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "lbhub-bridge.mib");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "lbhub-ecs.mib");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "lbhub-fms.mib");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "lbhub-mibii.mib");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "lbhub-msh.mib");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "lbhub-msh4ptbrdg.mib");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "lbhub-rblc.mib");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "lbhub-repeater.mib");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "linkb-mib");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "linkb-opt-fddi-mib");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "linkconv.mib");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "linkswitch1000.mib");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "llcmib");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "mipmib");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "mlnmib");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "ncdchass-mib");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "sdlc.mib");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "secmib.mib");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "startek");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "statusview.mib");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "switch2000tr-mib");
	}
}

sub dumpOID {
	open (STDOUT,">$NMIS::config{mib_root}/$ARGS{outfile}") or die "ERROR, problem opening $NMIS::config{mib_root}/$ARGS{outfile}. $!\n";

	SNMP_MIB::dump_oids_file();

	close (STDOUT);
}

exit(1);

